"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.yaml2ps = exports.ps2js = exports.js2ps = void 0;
const deps_js_1 = require("./deps.js");
const data = __importStar(require("./data.js"));
/**
 * Convert a JavaScript value into a PlatformScript value
 */
function js2ps(value) {
    let type = typeof value;
    if (type === "number") {
        return { type, value: value };
    }
    else if (type === "boolean") {
        return { type, value: value };
    }
    else if (type === "string") {
        return { type, value: value };
    }
    else if (type === "object") {
        if (Array.isArray(value)) {
            return { type: "list", value: value.map(js2ps) };
        }
        else {
            if (typeof value === "undefined") {
                throw new Error(`'undefined' cannot be converted into a PlatformScript value`);
            }
            let record = value;
            return {
                type: "map",
                value: new Map(Object.keys(record).flatMap((key) => {
                    if (typeof key === "string") {
                        let pskey = { type: "string", value: key };
                        let entry = [pskey, js2ps(record[key])];
                        return [entry];
                    }
                    else {
                        return [];
                    }
                })),
            };
        }
    }
    else {
        throw new Error(`cannot convert JavaScript value: '${value}' into PlatformScript`);
    }
}
exports.js2ps = js2ps;
/**
 * Convert a PlatformScript value into a JavaScript value
 */
function ps2js(value) {
    switch (value.type) {
        case "list":
            return value.value.map(ps2js);
        case "map":
            return [...value.value.entries()].reduce((obj, [k, v]) => {
                return Object.assign(obj, { [k.value.toString()]: ps2js(v) });
            }, {});
        case "fn":
            throw new Error("TODO: convert PlatformScript fn into callable JS fn");
        case "ref":
            throw new Error("TODO: de-reference YS references when converting to JS");
        default:
            return value.value;
    }
}
exports.ps2js = ps2js;
/**
 * Convert source YAML into a PlatformScript Literal
 */
function yaml2ps(node) {
    if (node.kind === 0) { // String, Number, Boolean
        let scalar = node;
        if (scalar.singleQuoted || scalar.doubleQuoted ||
            typeof scalar.valueObject === "undefined") {
            let expressions = matchTemplate(scalar.value);
            if (expressions.length > 0) {
                return createLiteral({
                    type: "template",
                    value: scalar.value,
                    expressions,
                }, node);
            }
            let match = matchReference(scalar.value);
            if (match) {
                return createLiteral({
                    type: "ref",
                    value: scalar.value,
                    key: match.key,
                    path: match.path,
                }, node);
            }
            return createLiteral({
                type: "string",
                value: scalar.value,
            }, node);
        }
        else {
            let value = scalar.valueObject;
            let type = typeof scalar.valueObject;
            if (!["number", "boolean"].includes(type)) {
                throw new Error(`unknown scalar type: ${type}`);
            }
            return createLiteral({ type: type, value }, node);
        }
    }
    else if (node.kind === 2) { // Map
        let mappings = node.mappings ?? [];
        let [first] = mappings;
        if (!first) {
            return { type: "map", value: new Map(), node };
        }
        else {
            let fnmatch = first.key.value.match(/^\(\s*(.*)\)\s*=>$/);
            if (fnmatch) {
                let param = fnmatch[1];
                let body = yaml2ps(first.value);
                return createLiteral({
                    type: "fn",
                    param: { name: param },
                    value: {
                        type: "platformscript",
                        body,
                    },
                }, node);
            }
            else {
                return createLiteral({
                    type: "map",
                    value: new Map(mappings.map((m) => {
                        let value = yaml2ps(m.value);
                        // use method syntax
                        let mmatch = m.key.value.match(/^\s*(.+)\((.*)\)\s*$/);
                        if (mmatch) {
                            let [, key, param] = mmatch;
                            return [data.string(key), {
                                    type: "fn",
                                    param: { name: param },
                                    value: {
                                        type: "platformscript",
                                        body: value,
                                    },
                                }];
                        }
                        else {
                            return [yaml2ps(m.key), value];
                        }
                    })),
                }, node);
            }
        }
    }
    else if (node.kind === 3) { // List
        let list = node;
        return createLiteral({
            type: "list",
            value: list.items.map(yaml2ps),
        }, node);
    }
    else {
        console.dir({ node }, { depth: 10 });
        throw new Error(`unknown YAMLNode of kind ${node.kind}`);
    }
}
exports.yaml2ps = yaml2ps;
function matchTemplate(value) {
    let valueIdx = 0;
    let exprIdx = 1;
    let regex = /%\(([\s\S]+?)\)/gmd;
    let i = value.matchAll(regex);
    let expressions = [];
    for (let next = i.next(); !next.done; next = i.next()) {
        let match = next.value;
        let expression = yaml2ps((0, deps_js_1.parseYAML)(match[exprIdx]));
        expressions.push({
            expression,
            //@ts-expect-error RegExpMatchArray#indices not yet in the default TS lib
            range: match.indices[valueIdx],
        });
    }
    return expressions;
}
function matchReference(value) {
    let pathIdx = 1;
    let match = value.match(/^\$(\S+)$/);
    if (match) {
        let [key, ...path] = match[pathIdx].split(".");
        return {
            key,
            path,
        };
    }
}
function createLiteral(value, node) {
    return Object.defineProperty(value, "node", {
        enumerable: false,
        value: node,
    });
}
//# sourceMappingURL=convert.js.map