"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPlatformScript = void 0;
const evaluate_js_1 = require("./evaluate.js");
const psmap_js_1 = require("./psmap.js");
const load_js_1 = require("./load.js");
const data_js_1 = require("./data.js");
const deps_js_1 = require("./deps.js");
function createPlatformScript(globals) {
    let env = lazy(() => {
        let ext = globals ? globals(platformscript) : (0, data_js_1.map)({});
        return (0, evaluate_js_1.createYSEnv)((0, psmap_js_1.concat)(evaluate_js_1.global, ext));
    });
    function run(block) {
        return (0, deps_js_1.run)(() => block(env()));
    }
    let platformscript = {
        run,
        call(fn, arg, options) {
            return run((env) => env.call(fn, arg, options));
        },
        eval(value, bindings) {
            return run((env) => env.eval(value, bindings));
        },
        moduleEval(value, url) {
            return run((env) => (0, load_js_1.moduleEval)({ body: value, location: url, env }));
        },
        load(location, base) {
            return run((env) => (0, load_js_1.load)({ location, base, env }));
        },
    };
    return platformscript;
}
exports.createPlatformScript = createPlatformScript;
function lazy(create) {
    let thunk = () => {
        let value = create();
        thunk = () => value;
        return value;
    };
    return () => thunk();
}
//# sourceMappingURL=platformscript.js.map